function createLEDImage, xsize, ysize, offImage=offImage, $
                         blackAndWhite=blackAndWhite

image8bit = bytscl(BESELJ(SHIFT(DIST(xsize,ysize),xsize/2,ysize/2)/6,0))
onImage = bytarr(3,xsize,ysize) ;+ 192B
indices = where(image8bit lt 99B)
image8bit[indices] = 192B
greyImage = bytarr(xsize,ysize)
greyImage[indices] = 192B
blackImage = 255B - image8Bit
blackImage[indices] = 192B
offImage = onImage
onImage[0,*,*] = greyImage
onImage[1,*,*] = image8bit
onImage[2,*,*] = greyImage

offImage[0,*,*] = image8bit
offImage[1,*,*] = greyImage
offImage[2,*,*] = greyImage

if keyword_set(blackAndWhite) then begin
  onImage[0,*,*] = image8bit
  onImage[1,*,*] = image8bit
  onImage[2,*,*] = image8bit

  offImage[0,*,*] = blackImage
  offImage[1,*,*] = blackImage
  offImage[2,*,*] = blackImage
endif

return, onImage
end

;{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|

pro krLEDDying,drawId

compile_opt hidden
if widget_info(drawId,/valid) then begin
  widget_control,drawId,get_uvalue=statePtr
  if obj_valid((*statePtr).oContainer) then obj_destroy,(*statePtr).oContainer
  if ptr_valid(statePtr) then ptr_free,statePtr
endif

return
end

;{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|

pro krLEDGetDrawId,drawId

compile_opt hidden
widget_control,drawId,get_value=visWin
widget_control,drawId,get_uvalue=statePtr
(*statePtr).oWindow=visWin
(*statePtr).oWindow->setCurrentCursor,'ORIGINAL'
(*statePtr).oContainer->add, visWin
;display the changes
(*statePtr).oWindow->Draw,(*statePtr).oView

return
end

;{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|

pro krLEDSetValue,base, onOff

drawId = widget_info(base,/child)
widget_control,drawId,get_uvalue=statePtr
(*statePtr).self->setProperty,onOff=onOff
(*statePtr).oWindow->Draw,(*statePtr).oView

return
end

;{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|

pro krLEDwidget::getProperty, wid=wid

if arg_present(wid) then wid=self.base

return
end

;{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|

pro krLEDwidget::setProperty, onOff=onOff

if strlowcase(onOff) eq 'on' then $
  self.oLEDImage->setProperty,data = *self.onImage $
    else self.oLEDImage->setProperty,data = *self.offImage

return
end

;{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|

pro krLEDwidget::cleanup


self->IDLgrMODEL::Cleanup
if ptr_valid(self.onImage) then ptr_free,self.onImage
if ptr_valid(self.offImage) then ptr_free,self.offImage

return
end

;{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|

function krLEDwidget::init, parentBase, onColor=onColor, $
              offColor=offColor, xsize=xsize, ysize=ysize, $
              LEDon=LEDon,title=title,  blackAndWhite=blackAndWhite, $
              _extra=extra

if n_elements(parentBase) eq 0 then begin
  message,'A parent base is required'
  return,0
endif

;initialize the super class
if (self->IDLgrModel::init(_extra=extra) ne 1) then return, 0

if not keyword_set(xsize) then xsize=20
if not keyword_set(ysize) then ysize=20
if not keyword_set(title) then title=''

;create the LED image
onImage = createLEDImage(xsize,ysize, offImage=offImage,blackAndWhite=blackAndWhite)
self.oLEDImage = obj_new('IDLgrImage', offImage)
oLEDImageModel = obj_new('IDLgrModel',select_target=1)
oLEDImageModel->add,self.oLEDImage
self->add,oLEDImageModel
self.onImage = ptr_new(onImage,/no_copy)
self.offImage = ptr_new(offImage,/no_copy)

self.base = widget_base(parentBase,_extra=extra, $
                        pro_set_value='krLEDSetValue');,space=0,ypad=0)

self.drawId = widget_draw(self.base, expose_events=1, $
            retain=0, graphics_level=2, xSize=xsize, YSize=ySize, $
            notify='krLEDGetDrawId', $
            kill_notify='krLEDDying',renderer=1)
if title ne '' then $
  self.textId = widget_label(self.base, value=title, _extra=extra)

;Create the coordinate system for the tree window.
oView = obj_new('IDLgrView', Viewplane_Rect=[0,0,xSize,ySize]) ;, $
                 ;ZCLIP=[100.0, -100.0]);, Color=[100,100,100])
;do this so that self gets displayed
oView->add,self

oContainer = Obj_New('IDL_Container')
oContainer->Add, oView
oContainer->add,self.oLEDImage
oContainer->add,oLEDImageModel

self.xsize = xsize
self.ysize = ysize

state = {self :self, $ ;object reference
         oView : oView, $
         oWindow : obj_new(), $
         oContainer : oContainer }

statePtr = ptr_new(state)
;have to store the pointer in the widgets
widget_control,self.drawId,set_uvalue=statePtr

return,1
end

;{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|

pro krLEDwidget__define
;defintion routine for the range meter object
;
; Copyright July, 2002
; Kling Research and Software, Inc.
; 7038 Westmoreland Dr.
; Warrenton, VA 20187
;

void = { krLEDwidget, $
         inherits IDLgrModel, $
         oLEDImage : obj_new(), $
         onImage : ptr_new(), $
         offImage : ptr_new(), $
         base : 0L, $ ;base widget id
         drawId : 0L, $ ;draw widget id
         textId : 0l, $ ;label widget id
         title : '', $ ;title field
         xsize : 0, $ ;xsize in pixels
         ysize : 0 };ysize in pixels

return
end

;{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|

function krLEDwidget, parentBase, objectRef=objectRef, _extra=extra

objectRef = obj_new('krledwidget',parentBase,_extra=extra)
if obj_valid(objectRef) then objectRef->getProperty,wid=wid else wid=-1L

return,wid
end

